@@ -14,6 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import fractions
 import functools
 import heapq
@@ -45,9 +46,21 @@ IMMEDIATE = 'immediate'
 class Watcher(object):
     """A **read-only** object representing a periodics callbacks activities."""
 
+    _REPR_MSG_TPL = ("<Watcher object at 0x%(ident)x "
+                     "("
+                     "runs=%(runs)s,"
+                     " successes=%(successes)s,"
+                     " failures=%(failures)s,"
+                     " elapsed=%(elapsed)0.2f,"
+                     " elapsed_waiting=%(elapsed_waiting)0.2f"
+                     ")>")
+
     def __init__(self, metrics):
         self._metrics = metrics
 
+    def __repr__(self):
+        return self._REPR_MSG_TPL % dict(ident=id(self), **self._metrics)
+
     @property
     def runs(self):
         """How many times the periodic callback has been ran."""
@@ -102,7 +115,17 @@ def _check_attrs(obj):
     return missing_attrs
 
 
-def periodic(spacing, run_immediately=False):
+def is_periodic(obj):
+    """Check whether an object is a valid periodic callable.
+
+    :param obj: object to inspect
+    :type obj: anything
+    :return: True if obj is a periodic task, otherwise False
+    """
+    return callable(obj) and not _check_attrs(obj)
+
+
+def periodic(spacing, run_immediately=False, enabled=True):
     """Tags a method/function as wanting/able to execute periodically.
 
     :param spacing: how often to run the decorated function (required)
@@ -111,6 +134,8 @@ def periodic(spacing, run_immediately=False):
                             immediately or wait until the spacing provided has
                             elapsed before running for the first time
     :type run_immediately: boolean
+    :param enabled: whether the task is enabled to run
+    :type enabled: boolean
     """
 
     if spacing <= 0:
@@ -118,7 +143,7 @@ def periodic(spacing, run_immediately=False):
                          " zero instead of %s" % spacing)
 
     def wrapper(f):
-        f._is_periodic = True
+        f._is_periodic = enabled
         f._periodic_spacing = spacing
         f._periodic_run_immediately = run_immediately
 
@@ -257,12 +282,8 @@ def _run_callback_no_retain(now_func, cb, *args, **kwargs):
 def _build(now_func, callables, next_run_scheduler):
     schedule = _Schedule()
     now = None
-    immediates = []
-    # Reverse order is used since these are later popped off (and to
-    # ensure the popping order is first -> last we need to append them
-    # in the opposite ordering last -> first).
-    reverse_it = utils.reverse_enumerate(callables)
-    for index, (cb, _cb_name, args, kwargs) in reverse_it:
+    immediates = collections.deque()
+    for index, (cb, _cb_name, args, kwargs) in enumerate(callables):
         if cb._periodic_run_immediately:
             immediates.append(index)
         else:
@@ -273,6 +294,31 @@ def _build(now_func, callables, next_run_scheduler):
     return immediates, schedule
 
 
+_SCHEDULE_RETRY_EXCEPTIONS = (RuntimeError, futurist.RejectedSubmission)
+
+
+class ExecutorFactory(object):
+    """Base class for any executor factory."""
+
+    shutdown = True
+    """Whether the executor should be shut down on periodic worker stop."""
+
+    def __call__(self):
+        """Return the executor to be used."""
+        raise NotImplementedError()
+
+
+class ExistingExecutor(ExecutorFactory):
+    """An executor factory returning the existing object."""
+
+    def __init__(self, executor, shutdown=False):
+        self._executor = executor
+        self.shutdown = shutdown
+
+    def __call__(self):
+        return self._executor
+
+
 class PeriodicWorker(object):
     """Calls a collection of callables periodically (sleeping as needed...).
 
@@ -295,6 +341,10 @@ class PeriodicWorker(object):
         'successes': 0,
     }
 
+    # When scheduling fails temporary, use a random delay between 0.9-1.1 sec.
+    _RESCHEDULE_DELAY = 0.9
+    _RESCHEDULE_JITTER = 0.2
+
     DEFAULT_JITTER = fractions.Fraction(5, 100)
     """
     Default jitter percentage the built-in strategies (that have jitter
@@ -344,7 +394,7 @@ class PeriodicWorker(object):
                log=None, executor_factory=None,
                cond_cls=threading.Condition, event_cls=threading.Event,
                schedule_strategy='last_started', now_func=utils.now,
-               on_failure=None):
+               on_failure=None, args=_NO_OP_ARGS, kwargs=_NO_OP_KWARGS):
         """Automatically creates a worker by analyzing object(s) methods.
 
         Only picks up methods that have been tagged/decorated with
@@ -366,7 +416,7 @@ class PeriodicWorker(object):
                                  provided one will be created that uses
                                  the :py:class:`~futurist.SynchronousExecutor`
                                  class)
-        :type executor_factory: callable
+        :type executor_factory: ExecutorFactory or any callable
         :param cond_cls: callable object that can
                           produce ``threading.Condition``
                           (or compatible/equivalent) objects
@@ -401,6 +451,10 @@ class PeriodicWorker(object):
                            any user provided callable should not raise
                            exceptions on being called
         :type on_failure: callable
+        :param args: positional arguments to be passed to all callables
+        :type args: tuple
+        :param kwargs: keyword arguments to be passed to all callables
+        :type kwargs: dict
         """
         callables = []
         for obj in objects:
@@ -410,10 +464,7 @@ class PeriodicWorker(object):
                 if six.callable(member):
                     missing_attrs = _check_attrs(member)
                     if not missing_attrs:
-                        # These do not support custom args, kwargs...
-                        callables.append((member,
-                                          cls._NO_OP_ARGS,
-                                          cls._NO_OP_KWARGS))
+                        callables.append((member, args, kwargs))
         return cls(callables, log=log, executor_factory=executor_factory,
                    cond_cls=cond_cls, event_cls=event_cls,
                    schedule_strategy=schedule_strategy, now_func=now_func,
@@ -445,7 +496,7 @@ class PeriodicWorker(object):
                                  provided one will be created that uses
                                  the :py:class:`~futurist.SynchronousExecutor`
                                  class)
-        :type executor_factory: callable
+        :type executor_factory: ExecutorFactory or any callable
         :param cond_cls: callable object that can
                           produce ``threading.Condition``
                           (or compatible/equivalent) objects
@@ -485,6 +536,7 @@ class PeriodicWorker(object):
         self._waiter = cond_cls()
         self._dead = event_cls()
         self._active = event_cls()
+        self._cond_cls = cond_cls
         self._watchers = []
         self._callables = []
         for (cb, args, kwargs) in callables:
@@ -530,6 +582,86 @@ class PeriodicWorker(object):
 
     def _run(self, executor, runner):
         """Main worker run loop."""
+        barrier = utils.Barrier(cond_cls=self._cond_cls)
+
+        def _process_scheduled():
+            # Figure out when we should run next (by selecting the
+            # minimum item from the heap, where the minimum should be
+            # the callable that needs to run next and has the lowest
+            # next desired run time).
+            with self._waiter:
+                while (not self._schedule and
+                       not self._tombstone.is_set() and
+                       not self._immediates):
+                    self._waiter.wait(self.MAX_LOOP_IDLE)
+                if self._tombstone.is_set():
+                    # We were requested to stop, so stop.
+                    return
+                if self._immediates:
+                    # This will get processed in _process_immediates()
+                    # in the next loop call.
+                    return
+                submitted_at = now = self._now_func()
+                next_run, index = self._schedule.pop()
+                when_next = next_run - now
+                if when_next <= 0:
+                    # Run & schedule its next execution.
+                    cb, cb_name, args, kwargs = self._callables[index]
+                    self._log.debug("Submitting periodic function '%s'",
+                                    cb_name)
+                    try:
+                        fut = executor.submit(runner,
+                                              self._now_func,
+                                              cb, *args, **kwargs)
+                    except _SCHEDULE_RETRY_EXCEPTIONS as exc:
+                        # Restart after a short delay
+                        delay = (self._RESCHEDULE_DELAY +
+                                 random().random() * self._RESCHEDULE_JITTER)
+                        self._log.error("Failed to submit periodic function "
+                                        "'%s', retrying after %.2f sec. "
+                                        "Error: %s",
+                                        cb_name, delay, exc)
+                        self._schedule.push(self._now_func() + delay,
+                                            index)
+                    else:
+                        barrier.incr()
+                        fut.add_done_callback(functools.partial(_on_done,
+                                                                PERIODIC,
+                                                                cb, cb_name,
+                                                                index,
+                                                                submitted_at))
+                        fut.add_done_callback(lambda _fut: barrier.decr())
+                else:
+                    # Gotta wait...
+                    self._schedule.push(next_run, index)
+                    when_next = min(when_next, self.MAX_LOOP_IDLE)
+                    self._waiter.wait(when_next)
+
+        def _process_immediates():
+            try:
+                index = self._immediates.popleft()
+            except IndexError:
+                pass
+            else:
+                cb, cb_name, args, kwargs = self._callables[index]
+                submitted_at = self._now_func()
+                self._log.debug("Submitting immediate function '%s'", cb_name)
+                try:
+                    fut = executor.submit(runner, self._now_func,
+                                          cb, *args, **kwargs)
+                except _SCHEDULE_RETRY_EXCEPTIONS as exc:
+                    self._log.error("Failed to submit immediate function "
+                                    "'%s', retrying. Error: %s", cb_name, exc)
+                    # Restart as soon as possible
+                    self._immediates.append(index)
+                else:
+                    barrier.incr()
+                    fut.add_done_callback(functools.partial(_on_done,
+                                                            IMMEDIATE,
+                                                            cb, cb_name,
+                                                            index,
+                                                            submitted_at))
+                    fut.add_done_callback(lambda _fut: barrier.decr())
 
         def _on_done(kind, cb, cb_name, index, submitted_at, fut):
             started_at, finished_at, failure = fut.result()
@@ -552,58 +684,12 @@ class PeriodicWorker(object):
                 self._schedule.push(next_run, index)
                 self._waiter.notify_all()
 
-        while not self._tombstone.is_set():
-            if self._immediates:
-                # Run & schedule its next execution.
-                try:
-                    index = self._immediates.pop()
-                except IndexError:
-                    pass
-                else:
-                    cb, cb_name, args, kwargs = self._callables[index]
-                    submitted_at = self._now_func()
-                    self._log.debug("Submitting immediate function '%s'",
-                                    cb_name)
-                    fut = executor.submit(runner,
-                                          self._now_func,
-                                          cb, *args, **kwargs)
-                    fut.add_done_callback(functools.partial(_on_done,
-                                                            IMMEDIATE,
-                                                            cb, cb_name,
-                                                            index,
-                                                            submitted_at))
-            else:
-                # Figure out when we should run next (by selecting the
-                # minimum item from the heap, where the minimum should be
-                # the callable that needs to run next and has the lowest
-                # next desired run time).
-                with self._waiter:
-                    while (not self._schedule and
-                           not self._tombstone.is_set()):
-                        self._waiter.wait(self.MAX_LOOP_IDLE)
-                    if self._tombstone.is_set():
-                        break
-                    submitted_at = now = self._now_func()
-                    next_run, index = self._schedule.pop()
-                    when_next = next_run - now
-                    if when_next <= 0:
-                        # Run & schedule its next execution.
-                        cb, cb_name, args, kwargs = self._callables[index]
-                        self._log.debug("Submitting periodic function '%s'",
-                                        cb_name)
-                        fut = executor.submit(runner,
-                                              self._now_func,
-                                              cb, *args, **kwargs)
-                        fut.add_done_callback(functools.partial(_on_done,
-                                                                PERIODIC,
-                                                                cb, cb_name,
-                                                                index,
-                                                                submitted_at))
-                    else:
-                        # Gotta wait...
-                        self._schedule.push(next_run, index)
-                        when_next = min(when_next, self.MAX_LOOP_IDLE)
-                        self._waiter.wait(when_next)
+        try:
+            while not self._tombstone.is_set():
+                _process_immediates()
+                _process_scheduled()
+        finally:
+            barrier.wait()
 
     def _on_finish(self):
         # TODO(harlowja): this may be to verbose for people?
@@ -696,7 +782,8 @@ class PeriodicWorker(object):
         try:
             self._run(executor, runner)
         finally:
-            executor.shutdown()
+            if getattr(self._executor_factory, 'shutdown', True):
+                executor.shutdown()
             self._dead.set()
             self._active.clear()
             self._on_finish()
@@ -718,6 +805,9 @@ class PeriodicWorker(object):
         self._dead.clear()
         for cb_metrics, _watcher in self._watchers:
             for k in list(six.iterkeys(cb_metrics)):
+                # NOTE(harlowja): mutate the original dictionaries keys
+                # so that the watcher (which references the same dictionary
+                # keys) is able to see those changes.
                 cb_metrics[k] = 0
         self._immediates, self._schedule = _build(
             self._now_func, self._callables, self._initial_schedule_strategy)
