@@ -18,6 +18,7 @@ import testscenarios
 from testtools import testcase
 
 import futurist
+from futurist import rejection
 from futurist.tests import base
 
 
@@ -79,7 +80,13 @@ class TestExecutors(testscenarios.TestWithScenarios, base.TestCase):
 
         self.assertEqual(3, self.executor.statistics.executed)
         self.assertEqual(1, self.executor.statistics.failures)
-        self.assertGreaterEqual(self.executor.statistics.runtime, 0.2)
+        self.assertGreaterEqual(self.executor.statistics.runtime,
+                                # It appears that the the thread run loop
+                                # may call this before 0.2 seconds (or 0.2
+                                # will not be represented as a float correctly)
+                                # is really up so accommodate for that
+                                # happening...
+                                0.199)
 
     def test_post_shutdown_raises(self):
         executor = self.executor_cls(**self.executor_kwargs)
@@ -131,3 +138,31 @@ class TestExecutors(testscenarios.TestWithScenarios, base.TestCase):
         self.assertEqual(10, len(happy_completed) + len(unhappy_completed))
         self.assertEqual(5, len(unhappy_completed))
         self.assertEqual(5, len(happy_completed))
+
+
+_REJECTION = rejection.reject_when_reached(1)
+
+
+class TestRejection(testscenarios.TestWithScenarios, base.TestCase):
+    scenarios = [
+        ('green', {'executor_cls': futurist.GreenThreadPoolExecutor,
+                   'executor_kwargs': {'check_and_reject': _REJECTION,
+                                       'max_workers': 1}}),
+        ('thread', {'executor_cls': futurist.ThreadPoolExecutor,
+                    'executor_kwargs': {'check_and_reject': _REJECTION,
+                                        'max_workers': 1}}),
+    ]
+
+    def setUp(self):
+        super(TestRejection, self).setUp()
+        self.executor = self.executor_cls(**self.executor_kwargs)
+
+    def test_rejection(self):
+        self.addCleanup(self.executor.shutdown)
+
+        # 1 worker + 1 item of backlog
+        for _i in range(2):
+            self.executor.submit(delayed, 0.5)
+
+        self.assertRaises(futurist.RejectedSubmission,
+                          self.executor.submit, returns_one)
